/*
 * The MIT License
 *
 * Copyright 2014 Mark A. Heckler
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.autonomous4j.listeners.xyz;

import com.dronecontrol.droneapi.listeners.VideoDataListener;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.autonomous4j.interfaces.A4jPublisher;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;

/**
 *
 * @author Mark Heckler (mark.heckler@gmail.com, @mkheck)
 */
public class A4jVideoDataListener implements A4jPublisher, VideoDataListener {
    private final static String TOP_LEVEL_TOPIC = "a4jvideodata";
    private MqttClient client;
    private final MqttMessage msg;
    private BufferedImage img;

    public A4jVideoDataListener() {
        msg = new MqttMessage();
        msg.setQos(0); // Deliver at most once (fire & forget) - crucial for video feed
        
        try{
        	
            client = new MqttClient("tcp://128.93.64.1:1883", MqttClient.generateClientId());
            client.connect();
            
        } catch (MqttException ex){
        	
            Logger.getLogger(A4jVideoDataListener.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    @Override
    public String getTopLevelTopic() {
        return TOP_LEVEL_TOPIC;
    }

    @Override
    public void publish() {
        try{
        	
            msg.setPayload(((DataBufferByte) img.getData().getDataBuffer()).getData());
            msg.setQos(0);
            client.publish(TOP_LEVEL_TOPIC+"/image", msg);
            System.out.println("Publish image "+msg.getPayload().length+" to topic "+TOP_LEVEL_TOPIC + "/image");
            
        } catch (MqttException ex) {
            Logger.getLogger(A4jVideoDataListener.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    @Override
    public void onVideoData(BufferedImage bi) {
        img = bi;
        publish();
    }
    
}
